/*	fuelAndIgnitionCalcs.c

	Copyright 2008 Fred Cooke

	This file is part of the FreeEMS project.

	FreeEMS software is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	FreeEMS software is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with any FreeEMS software.  If not, see <http://www.gnu.org/licenses/>.

	We ask that if you make any changes to this file you send them upstream to us at admin@diyefi.org

	Thank you for choosing FreeEMS to run your engine! */

#define FUELANDIGNITIONCALCS_C
#include "inc/freeEMS.h"
#include "inc/fuelAndIgnitionCalcs.h"

/* Based on the latest ADC readings, determine pulsewidth, dwell, and advance for all channels */
void calculateFuelAndIgnition(){
	/*&&&&&&&&&&&&&&&&& Set up the banks and offsets for us to use for input and output &&&&&&&&&&&&&&&&&&*/
	/* Set up the banks for input and output */
	unsigned char localInputBank = outputBank;
	unsigned char localOutputBank = inputBank;
	/* Set up local output bank offsets */
	unsigned char localInjectionOffset;
	unsigned char localIgnitionOffset;
	if(localOutputBank){
		localInjectionOffset = 0;
		localIgnitionOffset = 0;
	}else{
		localInjectionOffset = INJECTION_CHANNELS;
		localIgnitionOffset = IGNITION_CHANNELS;
	}
	/*&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&*/


	/*&&&&&&& Calculate and obtain the basic variables with which we will perform the calculations &&&&&&&*/
	
	// get IAT from ADC
	localIAT = IATTransferTable[ADCArrays.IAT[localInputBank]];
	
	// get CHT from ADC
	localCHT = CHTTransferTable[ADCArrays.CHT[localInputBank]];
	
	// get MAP from ADC
	localMAP = (((unsigned long)ADCArrays.MAP[localInputBank] * MAPRange) / ADC_DIVISIONS) + MAPMinimum;

	// get AAP from ADC
	localAAP = (((unsigned long)ADCArrays.AAP[localInputBank] * AAPRange) / ADC_DIVISIONS) + AAPMinimum;

	// get EGO from ADC
	localEGO = (((unsigned long)ADCArrays.EGO[localInputBank] * EGORange) / ADC_DIVISIONS) + EGOMinimum;
	
	// get BRV from ADC
	localBRV = (((unsigned long)ADCArrays.BRV[localInputBank] * BRVRange) / ADC_DIVISIONS) + BRVMinimum;
	
	/* Bound the TPS ADC reading and shift it to start at zero */
	unsigned short localTPSADC = ADCArrays.TPS[localInputBank];
	if(localTPSADC < TPSMinimumADC){
		localTPSADC = 0;
	}else if(localTPSADC > TPSMaximumADC){
		localTPSADC = TPSRangeADC;
	}else{
		localTPSADC -= TPSMinimumADC;
	}
	
	// get TPS from ADC
	localTPS = ((unsigned long)localTPSADC * TPS_RANGE) / TPSRangeADC;
	// No need to add TPS min as  we know it is 0 by definition.
	
	// get spare ADC
	localSP = ADCArrays.SpareADC[localInputBank] << 6;
	/*&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&*/

	
	/*&&&&&&&&&&&& Perform the basic calculations one step at a time to get a final pulsewidth &&&&&&&&&&&*/	
	/*&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&*/

	
	/*&&&&&&&&&&&&& Based on EPW, FPW and IDT schedule the EPW start such that FPW is correct &&&&&&&&&&&&*/
	/*&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&*/

	
	/*&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& Calculate Dwell and Ignition angle &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&*/
	/*&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&*/

	
	/*&&&&&&&&&&&&&& Based on Dwell and Ignition angle schedule the start and end of dwell &&&&&&&&&&&&&&&*/
	/*&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&*/

	
	/*&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& TEMPORARY &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&*/

	/* "Calculate" the nominal total pulse width before per channel corrections */
	masterPulseWidth = (ADCArrays.EGO[localInputBank] << 6) + (ADCArrays.MAP[localInputBank] >> 4);

	/* "Calculate" the individual fuel pulse widths */
	unsigned char channel;
	for(channel = 0; channel < INJECTION_CHANNELS; channel++){
		injectorMainPulseWidths[channel + localInjectionOffset] = masterPulseWidth;
	}

	// TODO x 6 main, x 6 staged, x 6 flags for staged if(coreSettingsA & STAGED_ON){}
	
	/* Set the staged status on or off (for now based on changeable settings) */
	if(coreSettingsA & STAGED_ON){
		coreStatusA |= STAGED_REQUIRED;
	}else{
		coreStatusA &= ~STAGED_REQUIRED;
	}
	
	// temporary ign tests
	unsigned short intendedAdvance = ADCArrays.SpareADC[localInputBank] << 6;
	unsigned short intendedDwell = intendedAdvance >> 1;

	short c;
	for(c=0;c<IGNITION_CHANNELS;c++){
		ignitionAdvances[IGNITION_CHANNELS + localIgnitionOffset] = intendedAdvance;
	}
	currentDwell[localOutputBank] = intendedDwell;
	
//	unsigned short minPeriod = ignitionMinimumDwell << 1;
	//	if(intendedDwell < ignitionMinimumDwell){
//		dwellLength = ignitionMinimumDwell;
//	}else{
//		dwellLength = intendedDwell;
//	}
//	if(intendedPeriod < minPeriod){
//		dwellPeriod = minPeriod;
//	}else{
//		dwellPeriod = intendedPeriod;
//	}
//	PITLD0 = dwellPeriod;
	
	/* Calculate the fuel advances */
	// TODO x 6
	// just use one for all for now
	totalAngleAfterReferenceInjection = (ADCArrays.TPS[localInputBank] << 6);
	
	/* Calculate the dwell period */
	// TODO x 1
	
	/* Calculate the ignition advances */
	//TODO x 12
}
