/*	main.c

	Copyright 2008 Fred Cooke

	This file is part of the FreeEMS project.

	FreeEMS software is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	FreeEMS software is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with any FreeEMS software.  If not, see <http://www.gnu.org/licenses/>.

	We ask that if you make any changes to this file you send them upstream to us at admin@diyefi.org

	Thank you for choosing FreeEMS to run your engine! */

#define MAIN_OR_GLOBALS
#include "inc/main.h"

int  main(){
	// Set everything up.
	init();
		
	// Run forever repeating.
	while(TRUE){
	//	unsigned short start = realTimeCounterMillis;
		/* If ADCs require forced sampling, sample now */
		if(coreStatusA & FORCE_READING){
			/* Atomic block to ensure a full set of readings are taken together */
			ATOMIC_START();
			
			/* Check to ensure that a reading wasn't take before we entered a non interruptable state */
			if(coreStatusA & FORCE_READING){ // do we still need to do this

				sampleLoopADC(&ADCArrays);
//				sampleEachADC(&ADCArrays);
				timeOutADCreadingCounter++;
				
				/* Set flag to say calc required */
				coreStatusA |= CALC_FUEL_IGN;

				/* Clear force reading flag */
				coreStatusA &= ~FORCE_READING;
			}

			ATOMIC_END();
		}

		/* If required, do main fuel and ignition calcs first */
		if(coreStatusA & CALC_FUEL_IGN){
			/* Atomic block to ensure that we don't clear the flag for the next data set when things are tight */
			ATOMIC_START();
			
			/* Switch input bank so that we have a stable set of variables to read from */
			if(inputBank){
				inputBank = 0;
			}else{
				inputBank = 1;
			}
			
			/* Clear the calc required flag */
			coreStatusA &= ~CALC_FUEL_IGN;

			ATOMIC_END();
			
			/* Perform the calculations TODO possibly move this to the software interrupt if it makes sense to do so */
			calculateFuelAndIgnition();
			
			/* Atomic block to ensure that outputBank and outputBank Offsets match */
			ATOMIC_START();
			
			/* Switch output bank to latest */
			if(outputBank){
				outputBank = 0;
				outputBankInjectionOffset = 0;
				outputBankIgnitionOffset = 0;
			}else{
				outputBank = 1;
				outputBankInjectionOffset = INJECTION_CHANNELS;
				outputBankIgnitionOffset = IGNITION_CHANNELS;
			}
			
			ATOMIC_END();
			
			/* Allow outputs to be read from */
			coreStatusA |= CALCS_VALID;
		}

		// Measure run time :
	//	unsigned short stop = realTimeCounterMillis; 
	//	unsigned short runtime = stop - start;
		
		unsigned short secondsBuffer = realTimeCounterSeconds;
		
		if(lastSeconds != secondsBuffer){
			// send logging once per second
			sendHRLog();
			
			// mechanism to ensure we send once per second without doing it in the RTC section.
			lastSeconds = secondsBuffer;
		}
		
		// Do not remove until there is sufficient code in here to make the main loop take a while
		sleep(5); // Ignoring this causes interrupts to be locked out far far too often.
		
		// on once per cycle for J0 (tacho)
		PORTJ ^= 0x01;

		// PWM experimentation
		adjustPWM();
	}
}
