/*	realtimeISRs.c

	Copyright 2008 Fred Cooke

	This file is part of the FreeEMS project.

	FreeEMS software is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	FreeEMS software is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with any FreeEMS software.  If not, see <http://www.gnu.org/licenses/>.

	We ask that if you make any changes to this file you send them upstream to us at admin@diyefi.org

	Thank you for choosing FreeEMS to run your engine! */

#define REALTIMEISRS_C
#include "inc/freeEMS.h"
#include "inc/interrupts.h"
#include "inc/realtimeISRs.h"

/* Real Time Interrupt Interrupt Service Routine */
/* Handles time keeping and generic periodic tasks that run quickly and must be done on time. */
void RTIISR(void)
{
	/* Increment the counter */
	realTimeCounterMain++;

	/* This function could be performed without the extra variables by rolling over the main ones at the largest multiples of the next ones, but I'm not sure thats better */

	/* Perform all tasks that are once per tenth of a second here or preferrably main */
	// TODO add content to eighths of a milli RTC ?

	/* Every 8th RTI execution is one milli */
	if(realTimeCounterMain % 8 == 0){
		/* Increment the milli counter */
		realTimeCounterMillis++;

		/* Increment the milli roll over variable */
		millisToTenths++;

		/* Perform all tasks that are once per millisecond here or preferrably main */
		ADCreadingTOCounter++;
		if(ADCreadingTOCounter > readingTimeout){
			/* Set force read adc flag */
			coreStatusA |= FORCE_READING;
			ADCreadingTOCounter = 0;
		}

		/* Every 100 millis is one tenth */
		if(millisToTenths % 100 == 0){
			/* Increment the tenths counter */
			realTimeCounterTenths++;

			/* Increment the tenths roll over variable */
			tenthsToSeconds++;

			/* Reset the millis roll over variable */
			millisToTenths = 0;

			/* Perform all tasks that are once per tenth of a second here or preferrably main */
			// decrement port H debounce variable till it's zero again.
			if(portHDebounce != 0){
				portHDebounce -= 1;
			}	

			/* Every 10 tenths is one second */
			if(tenthsToSeconds % 10 == 0){
				/* Increment the seconds counter */
				realTimeCounterSeconds++;

				/* Increment the seconds roll over variable */
				secondsToMinutes++;

				/* Reset the tenths roll over variable */
				tenthsToSeconds = 0;
				/* Perform all tasks that are once per second here or preferrably main */
				
				/* Every 60 seconds is one minute, 65535 minutes is enough for us :-) */
				if(secondsToMinutes % 60 == 0){
					/* Increment the minutes counter */
					realTimeCounterMinutes++;

					/* Potentially put an hours field in here and below, but that would be excessive */
					// TODO add hours RTC ?

					/* Reset the seconds roll over variable */
					secondsToMinutes = 0;

					/* Perform all tasks that are once per minute here or preferrably main */
					// TODO add content in minutes RTC ?
					
					/* Hours if statement here if we do hours which we probably won't */
				}
			}
		}
	}
	/* Clear the RTI flag */
	CRGFLG = 0x80; //7F;
}

/*  */
void ModDownCounter(void)
{
	/* Clear the modulus down counter interrupt flag */
	MCFLG = 0x80;

	/* Toggle the "user led" to indicate that the board is alive for new users */
	PORTP ^= 0x80;
}

/* When the ECT free running timer hits 65535 and rolls over, this is run */
void TimerOverflow(void){
	/* Increment the timer extension variable */
	timerExtension += 1; // see header declaration for idea to maybe speed up the other parts of this process TODO

	/* Clear the timer overflow interrupt flag */
	TFLGOF = 0x80;
}

/* Generic periodic interrupt (I think this only works from wait mode...) */
//void VRegAPIISR(void){
//	/* Clear the flag needs check because writing a 1 can set this one */
//	//if(VREGAPICL & 0x01){ // if the flag is set...
//		VREGAPICL |= 0x01; // clear it...
//	//} // and not otherwise!
//	
//	/* Flash the "user led" PWM 7 at 0.5Hz such that new users can confirm that their board is functioning properly */
//	PORTP ^= 0x80;
//}
